#########################################################
# J.Sanchez. Time Series for Data Scientists. CUP 2023  # 
#  Chapter 4  Case Study                                #
# program up to and including Figure 4.4                #
#                                                       #
#########################################################



##############Read the data #################
# set working directory to where the data is 

  park <- read.csv("dataset.csv")
  attach(park)
  head(park)
  tail(park)


#######################################################
# Section 4.7.2 Inspecting the quality of the data    #
#######################################################


##### Just testing how many are negative percent or 
### beyond 100% occupancy rate 
  occupancy=park$Occupancy; capacity=park$Capacity
  test= park 
  test$percent= occupancy/capacity
  head(test)
  
## Some observations have occupancy rate % > 100
## That is not good 
  
  badpercent=test[test$percent>1,]
  dim(badpercent) # how many observations like that 
  
  table(badpercent$SystemCodeNumber)
  
## Some observations have occupancy rate <0. Not good. 
  
  badmin=test[test$percent< 0,]
  dim(badmin)
  table(badmin$SystemCodeNumber)


## Investigate parks with less than 1312 observations. 
## for example
  park[SystemCodeNumber=="Bull Ring",]


########## Back to the original data 

## We check whether the number of observations differ by park
  table(SystemCodeNumber)
  dim(park)
 ## they vary
  
## We check whether there are explicit NAs. 
  ## There are missing observations, but not denoted by NA

  sum(is.na(park[, 1]))
  sum(is.na(park[, 2]))
  sum(is.na(park[, 3]))
  sum(is.na(park[, 4]))

# There are no NA's explicitly, so we explored the data more.
  ## Many places do not indicate a missing value with NA, like R does. 
## So missing values (missing dates) must be explore in different ways. 

  park[park$Occupancy <= 0, ]

# 1) Parking lots with SystemCodeNumber "BHMBCCPST01" and "BHMBRCBRG01" 
# both had 7 0-values.
# "NIA North" had values that were less than or equal 0. While a parking
# lot can have 0 occupied spots (i.e. it's empty), having negative occupancy
# isn't logical.

# 2) Suspect dates:

  park[park$SystemCodeNumber == "BHMBCCPST01" & park$Occupancy <= 0, ]

# BHMBCCPST01 had 2 "0 occupancy days" on 10/22/16
# Total: 2 0-values on 1 day.

  park[park$SystemCodeNumber == "BHMBRCBRG01" & park$Occupancy <= 0, ]

# BHMBRCBRG01 had 5 0-occupancy days across 11/22/16 and 11/23/16
# Total: 5 0-values on 1 day.

  park[park$SystemCodeNumber == "NIA North" & park$Occupancy <= 0, ]

# NIA North had 22 "0 or less than 0 occupancy days" across 10/16/16,
# 10/18/16, 10/28/16-10/30/16, 11/07/16, 11/10/17, and 11/30/16.
# Total: 22 <= 0-values across 8 days.

    park[park$Occupancy < 0, ]

    
    ## We will only count negative numbers as missing values,
# which there are 12 of.

### Cleaning the initial data set a little and finding means and medians

   park_clean <- park[park$SystemCodeNumber != "NIA North", ]
   park_clean <- droplevels(park_clean) # So NIA North ID's are dropped from the
# park_clean$SystemCodeNumber factor.

  dim(park_clean)
   head(park_clean)
   
# Create a variable with occupancy rate
# We do not multiply by 100 to represent percent. The 
# numbers are kept as fractions. 
   
   park_clean$Percent <- park_clean$Occupancy/park_clean$Capacity

  #############################################
  ## Prepare the information for Table 4.4    #
  #############################################

 x <- with(park_clean, tapply(Percent, SystemCodeNumber, mean))
 park_percent_summary <- data.frame(ID = names(x), mean = x, row.names = 1:29)
# Make data frame of Parking ID's in column 1 and average percent capacity in 
# colum 2. Row names set to number, instead of duplicate of Parking ID's.
   head(x)

  park_percent_summary$sd <- round(with(park_clean,
                    tapply(Percent, SystemCodeNumber, sd)),2)
  park_percent_summary$max <- round(with(park_clean,
                    tapply(Percent, SystemCodeNumber, max)),2)
  park_percent_summary$min <- round(with(park_clean,
                    tapply(Percent, SystemCodeNumber, min)),2)
  park_percent_summary$median <-round(with(park_clean,
                    tapply(Percent, SystemCodeNumber, median)),2)
  park_percent_summary$length <-round(with(park_clean,
                    tapply(Percent, SystemCodeNumber, length)),2)

  park_percent_summary
      head(park_percent_summary)

         #######################################################
         # Unsupervised machine learning: kmeans clustering    #
         #######################################################

##### Clustering of chosen parkings based on basic summary stats features. 

  ourcluster=kmeans(park_percent_summary[c(1:7,9:29),2:6],4)$cluster


## See which parks fall in which cluster. Table 4.4 
  view.which=data.frame(park_percent_summary[c(1:7,9:29),1:7], ourcluster)
  view.which

### Repeat and just keep what we need for plotting. 
  ch4clusterdata=data.frame(park_percent_summary[c(1:7,9:29),2:6], ourcluster)
  ch4clusterdata


           ######### Figure 4.3 ###########

## png("ch4birminghancluster.png", units="in", width=8.5, height=8, res=1200)


par(
  mfrow=c(2,1),
  font.axis=2, 
  mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
)

  plot(ch4clusterdata[,5], ch4clusterdata[,3],
     col=c("red","blue","purple","green")[unclass(ch4clusterdata[,6])],
     pch=c(16,17,18,19)[unclass(ch4clusterdata[,6])], 
     main="K-means clustering separates the parks well",
     ylim=c(0.4,1.1),xlab="Median occupancy", ylab="Max occupancy",
     lwd=1.5)
  text(ch4clusterdata[,5], ch4clusterdata[,3], labels=ch4clusterdata[,6],pos=1)


  plot(ch4clusterdata[,5], ch4clusterdata[,2],
     col=c("red","blue","purple","green")[unclass(ch4clusterdata[,6])],
     pch=c(16,17,18,19)[unclass(ch4clusterdata[,6])], 
     main="The clustering persists viewed with other dimension",
     ylim=c(0.05,0.37),xlab="Median occupancy", ylab="Standard deviation",
     lwd=1.5)
  text(ch4clusterdata[,5], ch4clusterdata[,2], labels=ch4clusterdata[,6],pos=1)

  #dev.off()  # make sure to run this once you see the plot

#### Be aware that kmeans changes the numbers of the clusters each time we
## execute it. So your cluster numbers will not be the same as the ones 
## resulting from the above plots. 

            
  ############################################################
  # Section 4.7.3 Selecting a few parking lots to study the  #
  # seasonalities                                              # 
  ############################################################


# To find which parking ID's we want to look at, we look at the number
# of observations for each ID and see that there are a few numbers that
# come up multiple times, ie a lot of ID's shared the same number of
# observations.  Six ID's that had 1312 observations are chosen, 
# making sure that they represent 
# high, medium, and low occupancy rates.

# SO, THE SELECTION IS NOT RANDOM.

  table(park$SystemCodeNumber) # See number of observations for each ID.

  test_lots <- c("Others-CCCPS135a", "Shopping", "Others-CCCPS8", "Broad Street",
               "Others-CCCPS98", "BHMBCCMKT01")

  ##### Prepare to reshape the data to prepare for Figure 4.4 #####


  sample_park <- park[park$SystemCodeNumber == test_lots[1], c(4, 2, 3)]
  head(sample_park)
  dim(sample_park)

for (i in seq_len(5)) {
  sample_park <- cbind(sample_park,
                       park[park$SystemCodeNumber == test_lots[i + 1], c(2, 3)])
  # cbind the Capacity and Occupancy columns extracted from the list of
  # Parking ID's chosen to be looked at.
}


   head(sample_park)

  colnames(sample_park) <- c("time_order")
  colnames(sample_park)

 for (i in seq_len(6)) {
  names(sample_park)[c(i * 2, i * 2 + 1)] <- paste(test_lots[i],
                                                   c("_capacity", "_occupancy"),
                                                   sep = "")
  
}

 head(sample_park)
 sample_park_percents <- sample_park[1]


  for (i in seq_len(6)) {
    sample_park_percents[i + 1] <- round(sample_park[, i * 2 + 1] / sample_park[, i * 2], 2)
  # Divide occupancy / capacity for every ID.
   }
  head(sample_park_percents)
   tail(sample_park_percents)

### Remember that although the variable is called percent, it is just the 
## fraction. Multiply by 100 to obtain percentage. 

##### Final data set to use
  names(sample_park_percents)[2:7] <- paste(test_lots, "_percent", sep = "")
  head(sample_park_percents)
   class(sample_park_percents)
   str(sample_park_percents)

##############################################################
##########  Preparing for Figure 4.4              #
#############################################################

#install.packages("lubridate")
   library(lubridate)


# Change the date from character to actual DATE. 
    sample_park_percents$date=ymd_hms(sample_park_percents$time_order)
   head(sample_park_percents)
    str(sample_park_percents)
   attach(sample_park_percents)
   class(date)  # Now date is a real date that R will read as a date. 
   finaldata=sample_park_percents[,2:8] 
   finaldata
   head(finaldata)
   attach(finaldata)

### Convert finaldata to xts object in order to do a
## nice time plot. First I test it with one variable
 #  install.packages("zoo")
   library("zoo")
   zoodata=zoo(finaldata[,1], order.by=finaldata[,7])
   plot.zoo(zoodata)
   head(zoodata)
     # good plot. I like it. Now I want it with the 6 parks. 

# install.packages("xts")
  library(xts)
  xtsdata=xts(finaldata[,1:6], order.by=finaldata[,7])
  names(xtsdata) <- c("CCPS135a", "Shopping", "CCCPS8", "Broad St.",
                      "CCPS98", "BHMBCCMKT01")

  plotxts_fix <- fix(plot.xts)
 environment(plotxts_fix) <- asNamespace("xts") #save window popping up

### IMPORTANT COMMENT: Had to use the above two lines of code to 
#remove the label in 
# the top right of a normal plot.xts plot. 
#The time title/label was being cut off,
# So I removed it for a more visually pleasing graph. 
 # The instructions to what I changed using
# The fix function can be found here:

# https://stackoverflow.com/questions/50051076/changes-in-plotting-an-xts-object


 #########Figure 4.4 
# png("activity_6.png", units = "in", width = 8.5, height = 6, res = 1200)
par(font = 2,
    font.lab = 2,
    font.axis = 2)
plotxts_fix(xtsdata, multi.panel = TRUE,
         grid.col = "grey40",
         ylim = c(0, 1.2))
# dev.off() # run this after you see the plot


### The following is a stacked barplot version of Figure 4.5. 
## Notice that we are stacking the proportions. That is why 
## the vertical axis goes to more than 1. The horizontal 
## axis are the hours of the day, from 8 AM to 4:30 PM. 

 
 days <- c("Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", 
           "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", 
           "Saturday", "Sunday", "Monday")
 
 par(mfrow = c(1, 7), mar = c(1, 2.5, 5,1), oma = c(0, 0, 1, 0))

 
 for (i in seq_len(7)) {
   barplot(xtsdata[(i * 18 - 17):(i * 18),], 
           col = (palette(gray(seq(0,1,len = 5)))), ylim = c(0, 4.5),
           xaxt = "n")
   if (i == 1) {
     mtext(side = 3, line = -3, outer = TRUE, 
           text = "(Parking Occupancy / Parking Capacity) of 6 Parking Structures Over Two Weeks", cex = 1.5, font = 2)
   }
   mtext(side = 3, line = -4, paste("10/", i + 3, "/16", sep = ""))
   mtext(side = 3, days[i], font = 2, line = -2.5, cex = 1.25)
 }

 
 ### When doing complex plots like these, the reader will appreciate having 
 ## a more mechanical way of visualizing the information. 
